@@ -1,11 +1,16 @@
 use strict;
 use warnings;
 package CPAN::Uploader;
-BEGIN {
-  $CPAN::Uploader::VERSION = '0.103000';
-}
 # ABSTRACT: upload things to the CPAN
-
+$CPAN::Uploader::VERSION = '0.103007';
+#pod =head1 ORIGIN
+#pod
+#pod This code is mostly derived from C<cpan-upload-http> by Brad Fitzpatrick, which
+#pod in turn was based on C<cpan-upload> by Neil Bowers.  I (I<rjbs>) didn't want to
+#pod have to use a C<system> call to run either of those, so I refactored the code
+#pod into this module.
+#pod
+#pod =cut
 
 use Carp ();
 use File::Basename ();
@@ -13,12 +18,31 @@ use File::Spec;
 use HTTP::Request::Common qw(POST);
 use HTTP::Status;
 use LWP::UserAgent;
+use File::HomeDir;
 
 my $UPLOAD_URI = $ENV{CPAN_UPLOADER_UPLOAD_URI}
-              || 'http://pause.perl.org/pause/authenquery';
+              || 'https://pause.perl.org/pause/authenquery';
 
+#pod =method upload_file
+#pod
+#pod   CPAN::Uploader->upload_file($file, \%arg);
+#pod
+#pod   $uploader->upload_file($file);
+#pod
+#pod Valid arguments are:
+#pod
+#pod   user       - (required) your CPAN / PAUSE id
+#pod   password   - (required) your CPAN / PAUSE password
+#pod   subdir     - the directory (under your home directory) to upload to
+#pod   http_proxy - uri of the http proxy to use
+#pod   upload_uri - uri of the upload handler; usually the default (PAUSE) is right
+#pod   debug      - if set to true, spew lots more debugging output
+#pod
+#pod This method attempts to actually upload the named file to the CPAN.  It will
+#pod raise an exception on error.
+#pod
+#pod =cut
 
-use Data::Dumper;
 sub upload_file {
   my ($self, $file, $arg) = @_;
 
@@ -32,11 +56,12 @@ sub upload_file {
   $self = $self->new($arg) if $arg;
 
   if ($arg->{dry_run}) {
+    require Data::Dumper;
     $self->log("By request, cowardly refusing to do anything at all.");
     $self->log(
       "The following arguments would have been used to upload: \n"
-      . '$self: ' . Dumper($self)
-      . '$file: ' . Dumper($file)
+      . '$self: ' . Data::Dumper::Dumper($self)
+      . '$file: ' . Data::Dumper::Dumper($file)
     );
   } else {
     $self->_upload($file);
@@ -46,16 +71,19 @@ sub upload_file {
 sub _ua_string {
   my ($self) = @_;
   my $class   = ref $self || $self;
-  my $version = $class->VERSION;
+  my $version = defined $class->VERSION ? $class->VERSION : 'dev';
 
   return "$class/$version";
 }
 
+sub uri { shift->{upload_uri} || $UPLOAD_URI }
+sub target { shift->{target} || 'PAUSE' }
+
 sub _upload {
   my $self = shift;
   my $file = shift;
 
-  $self->log("registering upload with PAUSE web server");
+  $self->log("registering upload with " . $self->target . " web server");
 
   my $agent = LWP::UserAgent->new;
   $agent->agent( $self->_ua_string );
@@ -92,7 +120,7 @@ sub _upload {
   );
 
   # Make the request to the PAUSE web server
-  $self->log("POSTing upload for $file");
+  $self->log("POSTing upload for $file to $uri");
   my $response = $agent->request($request);
 
   # So, how'd we do?
@@ -103,8 +131,8 @@ sub _upload {
   if ($response->is_error) {
     if ($response->code == RC_NOT_FOUND) {
       die "PAUSE's CGI for handling messages seems to have moved!\n",
-        "(HTTP response code of 404 from the PAUSE web server)\n",
-        "It used to be: ", $UPLOAD_URI, "\n",
+        "(HTTP response code of 404 from the ", $self->target, " web server)\n",
+        "It used to be: ", $uri, "\n",
         "Please inform the maintainer of $self.\n";
     } else {
       die "request failed with error code ", $response->code,
@@ -118,11 +146,21 @@ sub _upload {
       "----- RESPONSE END -------\n"
     );
 
-    $self->log("PAUSE add message sent ok [" . $response->code . "]");
+    $self->log($self->target . " add message sent ok [" . $response->code . "]");
   }
 }
 
 
+#pod =method new
+#pod
+#pod   my $uploader = CPAN::Uploader->new(\%arg);
+#pod
+#pod This method returns a new uploader.  You probably don't need to worry about
+#pod this method.
+#pod
+#pod Valid arguments are the same as those to C<upload_file>.
+#pod
+#pod =cut
 
 sub new {
   my ($class, $arg) = @_;
@@ -131,40 +169,73 @@ sub new {
   bless $arg => $class;
 }
 
+#pod =method read_config_file
+#pod
+#pod   my $config = CPAN::Uploader->read_config_file( $filename );
+#pod
+#pod This reads the config file and returns a hashref of its contents that can be
+#pod used as configuration for CPAN::Uploader.
+#pod
+#pod If no filename is given, it looks for F<.pause> in the user's home directory
+#pod (from the env var C<HOME>, or the current directory if C<HOME> isn't set).
+#pod
+#pod See L<cpan_upload/CONFIGURATION> for the config format.
+#pod
+#pod =cut
 
 sub read_config_file {
   my ($class, $filename) = @_;
 
-  unless ($filename) {
-    my $home  = $ENV{HOME} || '.';
+  unless (defined $filename) {
+    my $home  = File::HomeDir->my_home || '.';
     $filename = File::Spec->catfile($home, '.pause');
 
     return {} unless -e $filename and -r _;
   }
 
-  # Process .pause
-  open my $pauserc, '<', $filename
-    or die "can't open $filename for reading: $!";
+  my %conf;
+  if ( eval { require Config::Identity } ) {
+    %conf = Config::Identity->load($filename);
+    $conf{user} = delete $conf{username} unless $conf{user};
+  }
+  else { # Process .pause manually
+    open my $pauserc, '<', $filename
+      or die "can't open $filename for reading: $!";
 
-  my %from_file;
-  while (<$pauserc>) {
-    chomp;
-    next unless $_ and $_ !~ /^\s*#/;
+    while (<$pauserc>) {
+      chomp;
+      next unless $_ and $_ !~ /^\s*#/;
 
-    my ($k, $v) = /^\s*(\w+)\s+(.+)$/;
-    Carp::croak "multiple enties for $k" if $from_file{$k};
-    $from_file{$k} = $v;
+      my ($k, $v) = /^\s*(\w+)\s+(.+)$/;
+      Carp::croak "multiple enties for $k" if $conf{$k};
+      $conf{$k} = $v;
+    }
   }
 
-  return \%from_file;
+  return \%conf;
 }
 
+#pod =method log
+#pod
+#pod   $uploader->log($message);
+#pod
+#pod This method logs the given string.  The default behavior is to print it to the
+#pod screen.  The message should not end in a newline, as one will be added as
+#pod needed.
+#pod
+#pod =cut
 
 sub log {
   shift;
   print "$_[0]\n"
 }
 
+#pod =method log_debug
+#pod
+#pod This method behaves like C<L</log>>, but only logs the message if the
+#pod CPAN::Uploader is in debug mode.
+#pod
+#pod =cut
 
 sub log_debug {
   my $self = shift;
@@ -175,15 +246,18 @@ sub log_debug {
 1;
 
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 CPAN::Uploader - upload things to the CPAN
 
 =head1 VERSION
 
-version 0.103000
+version 0.103007
 
 =head1 METHODS
 
@@ -224,6 +298,8 @@ used as configuration for CPAN::Uploader.
 If no filename is given, it looks for F<.pause> in the user's home directory
 (from the env var C<HOME>, or the current directory if C<HOME> isn't set).
 
+See L<cpan_upload/CONFIGURATION> for the config format.
+
 =head2 log
 
   $uploader->log($message);
@@ -250,10 +326,9 @@ Ricardo SIGNES <rjbs@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Ricardo SIGNES.
+This software is copyright (c) 2014 by Ricardo SIGNES.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
